@@ -2,7 +2,6 @@
 #' structural variants
 #' #@export
 #setClass("BreakpointGRanges", contains="GRanges")
-
 #' Partner breakend for each breakend.
 #'
 #' @details
@@ -26,6 +25,28 @@ partner <- function(gr, selfPartnerSingleBreakends=FALSE) {
   .assertValidBreakpointGRanges(gr, allowSingleBreakends=selfPartnerSingleBreakends)
   return(gr[ifelse(selfPartnerSingleBreakends & is.na(gr$partner), names(gr), gr$partner),])
 }
+#' Determines whether this breakend has a valid partner in this
+#' GRanges
+#'
+#' @param gr GRanges object of SV breakends
+#' @param selfPartnerSingleBreakends treat single breakends as their own partner.
+#' @return True/False for each row in the breakpoint GRanges
+#' @examples
+#' #Subset to chromosome 6 intra-chromosomal events \code{vcf}
+#' vcf.file <- system.file("extdata", "COLO829T.purple.sv.ann.vcf.gz",
+#'   package = "StructuralVariantAnnotation")
+#' vcf <- VariantAnnotation::readVcf(vcf.file)
+#' gr <- breakpointRanges(vcf)
+#' gr <- gr[seqnames(gr) == "6"]
+#' # We now need to filter out inter-chromosomal events to ensure
+#' # our GRanges doesn't contain any breakpoints whose partner
+#' # has already been filtered out and no longer exists in the GRanges.
+#' gr <- gr[hasPartner(gr)]
+#'@export
+hasPartner <- function(gr, selfPartnerSingleBreakends=FALSE) {
+  return((is.na(gr$partner) & selfPartnerSingleBreakends) |
+    (!is.na(gr$partner) & gr$partner %in% names(gr)))
+}
 
 #' Finding overlapping breakpoints between two breakpoint sets
 #'
@@ -81,7 +102,7 @@ findBreakpointOverlaps <- function(query, subject, maxgap=-1L, minoverlap=0L, ig
   localhits = findOverlaps(query, subject, maxgap=maxgap, minoverlap=minoverlap, type="any", select="all", ignore.strand=ignore.strand)
   remotehits = findOverlaps(pquery, squery, maxgap=maxgap, minoverlap=minoverlap, type="any", select="all", ignore.strand=ignore.strand)
   ## duplicated() version:
-  #hits = Hits(c(queryHits(localhits), queryHits(remotehits)), c(subjectHits(localhits), subjectHits(remotehits)), nLnode=nLnode(localhits), nRnode=nRnode(localhits), sort.by.query=TRUE)
+  #hits = Hits(c(S4Vectors::queryHits(localhits), S4Vectors::queryHits(remotehits)), c(S4Vectors::subjectHits(localhits), S4Vectors::subjectHits(remotehits)), nLnode=nLnode(localhits), nRnode=nRnode(localhits), sort.by.query=TRUE)
   #hits = hits[duplicated(hits)]
   
   ## intersect() version:
@@ -95,20 +116,20 @@ findBreakpointOverlaps <- function(query, subject, maxgap=-1L, minoverlap=0L, ig
   #	dplyr::filter(!is.na(dplyr::lead(.$queryHits)) & !is.na(dplyr::lead(.$subjectHits)) & dplyr::lead(.$queryHits) == .$queryHits & dplyr::lead(.$subjectHits) == .$subjectHits)
   
   ## dplyr() exploiting the sorted nature of the findOverlaps():
-  #hits = Hits(c(queryHits(localhits), queryHits(remotehits)), c(subjectHits(localhits), subjectHits(remotehits)), nLnode=nLnode(localhits), nRnode=nRnode(localhits), sort.by.query=TRUE)
-  #queryLead  = dplyr::lead(queryHits(hits))
-  #querySubject  = dplyr::lead(queryHits(hits))
+  #hits = Hits(c(S4Vectors::queryHits(localhits), S4Vectors::queryHits(remotehits)), c(S4Vectors::subjectHits(localhits), S4Vectors::subjectHits(remotehits)), nLnode=nLnode(localhits), nRnode=nRnode(localhits), sort.by.query=TRUE)
+  #queryLead  = dplyr::lead(S4Vectors::queryHits(hits))
+  #querySubject  = dplyr::lead(S4Vectors::queryHits(hits))
   #hits = hits[
   #	!is.na(queryLead) &d
   #	!is.na(querySubject) &
-  #	queryLead == queryHits(hits) &
-  #	querySubject == subjectHits(hits)]
+  #	queryLead == S4Vectors::queryHits(hits) &
+  #	querySubject == S4Vectors::subjectHits(hits)]
   if (!is.null(sizemargin) && !is.na(sizemargin)) {
     # take into account confidence intervals when calculating event size
     callwidth <- .distance(query, pquery)
     truthwidth <- .distance(subject, squery)
-    callsize <- callwidth + (query$insLen %na% 0)
-    truthsize <- truthwidth + (subject$insLen %na% 0)
+    callsize <- callwidth + .replaceNa(query$insLen, 0)
+    truthsize <- truthwidth + .replaceNa(subject$insLen, 0)
     sizeerror <- .distance(
       IRanges::IRanges(start=callsize$min[S4Vectors::queryHits(hits)], end=callsize$max[S4Vectors::queryHits(hits)]),
       IRanges::IRanges(start=truthsize$min[S4Vectors::subjectHits(hits)], end=truthsize$max[S4Vectors::subjectHits(hits)])
@@ -126,6 +147,7 @@ findBreakpointOverlaps <- function(query, subject, maxgap=-1L, minoverlap=0L, ig
   return(hits)
 }
 # TODO: new function to annotate a Hits object with sizeerror, localbperror, and remotebperror
+#' @noRd
 .distance <- function(r1, r2) {
   return(data.frame(
     min=pmax(0, pmax(start(r1), start(r2)) - pmin(end(r1), end(r2))),
@@ -145,8 +167,10 @@ findBreakpointOverlaps <- function(query, subject, maxgap=-1L, minoverlap=0L, ig
 #' @param breakpointScoreColumn Query column defining a score for determining which query breakpoint
 #' is considered the best when countOnlyBest=TRUE.
 #' @examples
-#' truth_vcf = VariantAnnotation::readVcf(system.file("extdata", "na12878_chr22_Sudmunt2015.vcf", package = "StructuralVariantAnnotation"))
-#' crest_vcf = VariantAnnotation::readVcf(system.file("extdata", "na12878_chr22_crest.vcf", package = "StructuralVariantAnnotation"))
+#' truth_vcf = VariantAnnotation::readVcf(system.file("extdata", "na12878_chr22_Sudmunt2015.vcf", 
+#' package = "StructuralVariantAnnotation"))
+#' crest_vcf = VariantAnnotation::readVcf(system.file("extdata", "na12878_chr22_crest.vcf", 
+#' package = "StructuralVariantAnnotation"))
 #' caller_bpgr = breakpointRanges(crest_vcf)
 #' caller_bpgr$true_positive = countBreakpointOverlaps(caller_bpgr, breakpointRanges(truth_vcf),
 #'   maxgap=100, sizemargin=0.25, restrictMarginToSizeMultiple=0.5, countOnlyBest=TRUE)
@@ -192,7 +216,7 @@ countBreakpointOverlaps <- function(querygr, subjectgr, countOnlyBest=FALSE,
 #' vcf.file <- system.file("extdata", "gridss.vcf", package = "StructuralVariantAnnotation")
 #' bpgr <- breakpointRanges(VariantAnnotation::readVcf(vcf.file))
 #' pairgr <- breakpointgr2pairs(bpgr)
-#' rtracklayer::export(pairgr, con="example.bedpe")
+#' #rtracklayer::export(pairgr, con="example.bedpe")
 #' @return Pairs GRanges object suitable for export to BEDPE by rtracklayer
 #' @rdname pairs2breakpointgr
 #' @export
@@ -205,7 +229,7 @@ breakpointgr2pairs <- function(
   .assertValidBreakpointGRanges(bpgr, "Cannot convert breakpoint GRanges to Pairs: ", allowSingleBreakends=FALSE)
   
   if (is.null(bedpeName)) {
-    bedpeName = function(gr) { (gr$sourceId %null% gr$name) %null% names(gr) }
+    bedpeName = function(gr) { .replaceNull(.replaceNull(gr$sourceId, gr$name), names(gr)) }
   }
   if (is.null(firstInPair)) {
     firstInPair = function(gr) { seq_along(gr) < match(gr$partner, names(gr)) }
@@ -220,6 +244,7 @@ breakpointgr2pairs <- function(
   }
   return(pairgr)
 }
+#' @noRd
 .assertValidBreakpointGRanges <- function(bpgr, friendlyErrorMessage="", allowSingleBreakends=TRUE) {
   if (is.null(names(bpgr))) {
     stop(paste0(friendlyErrorMessage, "Breakpoint GRanges require names"))
@@ -313,8 +338,8 @@ pairs2breakpointgr <- function(
 extractBreakpointSequence <- function(gr, ref, anchoredBases, remoteBases=anchoredBases) {
 	localSeq <- extractReferenceSequence(gr, ref, anchoredBases, 0)
 	insSeq <- ifelse(strand(gr) == "-",
-					 as.character(Biostrings::reverseComplement(DNAStringSet(gr$insSeq %na% ""))),
-					 gr$insSeq %na% "")
+					 as.character(Biostrings::reverseComplement(DNAStringSet(.replaceNa(gr$insSeq,"")))),
+					 .replaceNa(gr$insSeq, ""))
 	remoteSeq <- as.character(Biostrings::reverseComplement(DNAStringSet(
 		extractReferenceSequence(partner(gr), ref, remoteBases, 0))))
 	return(paste0(localSeq, insSeq, remoteSeq))
@@ -353,6 +378,7 @@ extractReferenceSequence <- function(gr, ref, anchoredBases, followingBases=anch
 #' @param ref reference 
 #' @param position only 'middle' position is accepted.
 #' @return A constricted GRanges object.
+#' @noRd
 .constrict <- function(gr, ref=NULL,position="middle") {
 	isLower <- start(gr) < start(partner(gr))
 	# Want to call a valid breakpoint
@@ -387,10 +413,10 @@ extractReferenceSequence <- function(gr, ref, anchoredBases, followingBases=anch
 #' @param anchorLength Number of bases to consider for homology
 #' @param margin Number of additional reference bases include. This allows
 #'		for inexact homology to be detected even in the presence of indels.
-#' @param mismatch see Biostrings::pairwiseAlignment
-#' @param gapOpening see Biostrings::pairwiseAlignment
-#' @param gapExtension see Biostrings::pairwiseAlignment
-#' @param match see Biostrings::pairwiseAlignment
+#' @param mismatch see pwalign::pairwiseAlignment
+#' @param gapOpening see pwalign::pairwiseAlignment
+#' @param gapExtension see pwalign::pairwiseAlignment
+#' @param match see pwalign::pairwiseAlignment
 #' @return A dataframe containing the length of inexact homology between the 
 #' breakpoint sequence and the reference.
 #' @export
@@ -401,7 +427,7 @@ calculateReferenceHomology <- function(gr, ref,
 									   #match = 1, mismatch = -4, gapOpening = 6, gapExtension = 1, # bowtie2
 ) {
 	# shrink anchor for small events to prevent spanning alignment
-	aLength <- pmin(anchorLength, abs(gr$svLen) + 1) %na% anchorLength
+	aLength <- .replaceNa(pmin(anchorLength, abs(gr$svLen) + 1), anchorLength)
 	anchorSeq <- extractReferenceSequence(gr, ref, aLength, 0)
 	anchorSeq <- sub(".*N", "", anchorSeq)
 	# shrink anchor with Ns
@@ -416,14 +442,14 @@ calculateReferenceHomology <- function(gr, ref,
 	partnerIndex <- match(gr$partner, names(gr))
 
 	if (all(refseq=="") && all(varseq=="")) {
-		# Workaround of Biostrings::pairwiseAlignment bug
+		# Workaround of pwalign::pairwiseAlignment bug
 		return(data.frame(
 			exacthomlen=rep(NA, length(gr)),
 			inexacthomlen=rep(NA, length(gr)),
 			inexactscore=rep(NA, length(gr))))
 	}
 
-	aln <- Biostrings::pairwiseAlignment(varseq, refseq, type="local",
+	aln <- pwalign::pairwiseAlignment(varseq, refseq, type="local",
 										 substitutionMatrix=nucleotideSubstitutionMatrix(match, mismatch, FALSE, "DNA"),
 										 gapOpening=gapOpening, gapExtension=gapExtension, scoreOnly=FALSE)
 	ihomlen <- Biostrings::nchar(aln) - aLength - deletion(nindel(aln))[,2] - insertion(nindel(aln))[,2]
@@ -433,7 +459,7 @@ calculateReferenceHomology <- function(gr, ref,
 	# TODO: replace this with an efficient longest common substring function
 	# instead of S/W with a massive mismatch/gap penalty
 	penalty <- anchorLength * match
-	matchLength <- Biostrings::pairwiseAlignment(varseq, refseq, type="local",
+	matchLength <- pwalign::pairwiseAlignment(varseq, refseq, type="local",
 												 substitutionMatrix=nucleotideSubstitutionMatrix(match, -penalty, FALSE, "DNA"),
 												 gapOpening=penalty, gapExtension=0, scoreOnly=TRUE) / match
 	ehomlen <- matchLength - aLength
@@ -454,6 +480,7 @@ calculateReferenceHomology <- function(gr, ref,
 #' @param insSeq insert sequence of the GRanges.
 #' @param ref reference sequence of the GRanges.
 #' @return breakendAlt or breakpointAlt depending on whether the variant is partnered.
+#' @noRd
 .toVcfBreakendNotationAlt = function(gr, insSeq=gr$insSeq, ref=gr$REF) {
 	assertthat::assert_that(all(width(gr) == 1))
 	assertthat::assert_that(!is.null(insSeq))
@@ -510,18 +537,21 @@ breakpointGRangesToVCF <- function(gr, ...) {
 	    verbose = FALSE)
 
 }
-#' Type of simplest explaination of event. Possible types are:
+#' Type of simplest explanation of event. Possible types are:
 #' | Type | Description |
 #' | BND | Single breakend |
 #' | CTX | Interchromosomal translocation |
-#' | INV | Inversion. Note that both ++ and -- breakpoint will be classified as inversion regardless of whether the matching breakpoint actually exists |
+#' | INV | Inversion. |
 #' | DUP | Tandem duplication |
 #' | INS | Insertion |
 #' | DEL | Deletion |
 #' 
+#' Note that both ++ and -- breakpoint will be classified as inversions regardless of whether both breakpoint that consistitute an actual inversion exists or not
+#' 
 #' @param gr breakpoint GRanges object
-#' @param insertionLengthThreshold portion of inserted bases compared to total event size to be classified as an insertion. For example, a 5bp deletion with 5 inserted bases will be classified as an INS event.
-#' @return Type of simplest explaination of event
+#' @param insertionLengthThreshold portion of inserted bases compared to total event size to be classified as an insertion.
+#' For example, a 5bp deletion with 5 inserted bases will be classified as an INS event.
+#' @return Type of simplest explanation of event
 #' @export
 simpleEventType <- function(gr, insertionLengthThreshold=0.5) {
 	if (is.null(gr$partner)) {
@@ -538,7 +568,7 @@ simpleEventType <- function(gr, insertionLengthThreshold=0.5) {
 }
 #' Length of event if interpreted as an isolated breakpoint.
 #' @param gr breakpoint GRanges object
-#' @return Length of the simplest explaination of this breakpoint/breakend.
+#' @return Length of the simplest explanation of this breakpoint/breakend.
 #' @export
 simpleEventLength <- function(gr) {
 	if (is.null(gr$partner)) {
@@ -556,7 +586,7 @@ simpleEventLength <- function(gr) {
 #' sequence, this representational difference is problematic when comparing
 #' variant call sets.
 #' 
-#' WARNING: this method does not yet check that the inserted sequence actually matched the duplicated sequence.
+#' WARNING: this method does not check that the inserted sequence actually matched the duplicated sequence.
 #' @param query a breakpoint GRanges object
 #' @param subject a breakpoint GRanges object
 #' @param maxgap maximum distance between the insertion position and the duplication
@@ -584,13 +614,14 @@ findInsDupOverlaps <- function(query, subject, maxgap=-1L, maxsizedifference=0L)
 		qhits=c(qins_to_sdup$queryHits, sins_to_qdup$subjectHits),
 		shits=c(qins_to_sdup$subjectHits, sins_to_qdup$queryHits))
 	# add upper to upper match
-	bothhits = Hits(
+	bothhits = S4Vectors::Hits(
 		from=c(lowhits$qhits, pquery$ordinal[lowhits$qhits]),
 		to=c(lowhits$shits, psubject$ordinal[lowhits$shits]),
 		nLnode=length(query),
 		nRnode=length(subject))
 	return(bothhits)
 }
+#' @noRd
 .findOverlaps_queryIns_subjectDup <- function(query, subject, psubject , maxgap=-1L, maxsizedifference=0L) {
 	subject$HighEndPosition = end(psubject)
 	subject = subject[subject$set == "DUP" & subject$isLowBreakend]
@@ -599,10 +630,215 @@ findInsDupOverlaps <- function(query, subject, maxgap=-1L, maxsizedifference=0L)
 	start(subject) = start(subject) - 1
 	query = query[query$set == "INS" & query$isLowBreakend]
 	hits = findOverlaps(query, subject, maxgap=maxgap, ignore.strand=TRUE)
-	hits = hits[abs(query$sel[queryHits(hits)] - subject$sel[subjectHits(hits)]) <= maxsizedifference]
+	hits = hits[abs(query$sel[S4Vectors::queryHits(hits)] - subject$sel[S4Vectors::subjectHits(hits)]) <= maxsizedifference]
 	# TODO: filter by 
 	# Translate back to ordinals of what was passed in to us
 	return(data.frame(
-		queryHits=query$ordinal[queryHits(hits)],
-		subjectHits=subject$ordinal[subjectHits(hits)]))
+		queryHits=query$ordinal[S4Vectors::queryHits(hits)],
+		subjectHits=subject$ordinal[S4Vectors::subjectHits(hits)]))
+}
+#' Identifies potential transitive imprecise calls that can be explained by
+#' traversing multiple breakpoints.
+#' 
+#' Transitive calls are imprecise breakpoints or breakpoints with inserted sequence
+#' that can be explained by a sequence of breakpoints.
+#' That is, A-C calls in which additional sequence may be between A and C that
+#' can be explained by A-B-C.
+#' 
+#' @param transitiveGr a breakpoint GRanges object containing imprecise calls
+#' @param subjectGr breakpoints to traverse
+#' @param maximumImpreciseInsertSize
+#' Expected number of bases to traverse imprecise calls.
+#' @param minimumTraversedBreakpoints
+#' Minimum number of traversed breakpoints to consider a transitive
+#' @param maximumTraversedBreakpoints
+#' Maximum number of breakpoints to traverse when looking for an explanation of the transitive calls
+#' @param positionalMargin 
+#' Allowable margin of error when matching call positional overlaps.
+#' A non-zero margin allows for matching of breakpoint with imperfect homology.
+#' @param insertionLengthMargin
+#' Allowable difference in length between the inserted sequence and the traversed
+#' path length.
+#' Defaults to 50bp to allow for long read indel errors.
+#' @param insLen
+#' Integer vector of same length as `transitiveGr` indicating the number
+#' of bases inserted at the breakpoint.
+#' 
+#' Defaults to transitiveGr$insLen which will be present if the GRanges
+#' was loaded from a VCF using breakpointRanges()
+#' @param impreciseTransitiveCalls
+#' Boolean vector of same length as `transitiveGr` indicating which calls
+#' are imprecise calls. Defaults to calls with a non-zero interval size
+#' that have no homology.
+#' @param impreciseSubjectCalls
+#' Boolean vector of same length as `subjectGr` indicating which calls
+#' are imprecise calls. Defaults to calls with a non-zero interval size
+#' that have no homology.
+#' @param allowImprecise Allow traversal of imprecise calls.
+#' Defaults to FALSE as to prevent spurious results which skip
+#' some breakpoints when traversing multiple breakpoints
+#' E.g. An A-D transitive from an underlying A-B-C-D rearrangement
+#' will include A-B-D and A-C-D results if allowImprecise=TRUE.
+#' @return `DataFrame` containing the transitive calls traversed with the following columns:
+#' | column | meaning | 
+#' | ------ | ------- | 
+#' | transitive_breakpoint_name | Name of the transitive breakpoint a path was found for |
+#' | total_distance | Total length (in bp) of the path |
+#' | traversed_breakpoint_names | `CharacterList` of names of breakpoint traversed in the path |
+#' | distance_to_traversed_breakpoint | `IntegerList` of distances from start of path to end of traversing breakpoint |
+#' @export
+findTransitiveCalls <- function(
+		transitiveGr,
+		subjectGr,
+		maximumImpreciseInsertSize=700,
+		minimumTraversedBreakpoints=2,
+		maximumTraversedBreakpoints=6,
+		positionalMargin=8,
+		insertionLengthMargin=50,
+		insLen=transitiveGr$insLen,
+		impreciseTransitiveCalls=(transitiveGr$HOMLEN == 0 | is.null(transitiveGr$HOMLEN)) & start(transitiveGr) != end(transitiveGr),
+		impreciseSubjectCalls=(subjectGr$HOMLEN == 0 | is.null(subjectGr$HOMLEN)) & start(subjectGr) != end(subjectGr),
+		allowImprecise=FALSE) {
+	if (is.null(insLen)) {
+		stop("Missing insLen")
+	}
+	transitiveGr$.isImprecise = impreciseTransitiveCalls
+	transitiveGr$insLen = insLen
+	transitiveGr = transitiveGr[hasPartner(transitiveGr)]
+	transitiveGr$.isImprecise = transitiveGr$.isImprecise | partner(transitiveGr)$.isImprecise
+	transitiveGr$minimumTransitiveLength = ifelse(insLen > 0, pmax(0, insLen - insertionLengthMargin), 0)
+	transitiveGr$maximumTransitiveLength = ifelse(insLen > 0, insLen + insertionLengthMargin, ifelse(impreciseTransitiveCalls, maximumImpreciseInsertSize, 0))
+	transitiveGr = transitiveGr[transitiveGr$maximumTransitiveLength > 0]
+	transitiveGr = transitiveGr[hasPartner(transitiveGr)]
+	transitiveGr$ordinal = seq_len(length(transitiveGr))
+	transitiveGr$partnerOrdinal = partner(transitiveGr)$ordinal
+	if (!allowImprecise) {
+		subjectGr = subjectGr[!impreciseSubjectCalls]
+	}
+	subjectGr = subjectGr[hasPartner(subjectGr)]
+	# centre-align subject intervals to simplify the traversal logic
+	start(subjectGr) = (start(subjectGr) + end(subjectGr)) / 2
+	if (is.null(subjectGr$insLen)) {
+		warning("insLen field missing. Assuming all traversed breakpoints have no inserted sequence")
+		subjectGr$insLen = 0
+	}
+	subjectGr$insLen = .replaceNa(subjectGr$insLen, 0)
+	subjectGr$ordinal = seq_len(length(subjectGr))
+	subjectGr$partnerOrdinal = partner(subjectGr)$ordinal
+	# transitive breakpoint must occur within the confidence interval bounds
+	terminal_matches = as.data.frame(GenomicRanges::findOverlaps(transitiveGr, subjectGr, maxgap=positionalMargin, ignore.strand=FALSE)) %>%
+		dplyr::select(
+			terminalStartOrdinal=.data$queryHits,
+			transitiveOrdinal=.data$subjectHits) %>%
+		dplyr::mutate(
+			terminalEndOrdinal=transitiveGr$partnerOrdinal[.data$terminalStartOrdinal])
+	current_traversals = dplyr::inner_join(terminal_matches, terminal_matches, by=c("terminalStartOrdinal"="terminalEndOrdinal"), suffix=c(".start", ".end")) %>%
+		dplyr::select(
+			terminalStartOrdinal=.data$terminalStartOrdinal,
+			currentTraverseInOrdinal=.data$transitiveOrdinal.start,
+			endingTraverseOutOrdinal=.data$transitiveOrdinal.end,
+			terminalEndOrdinal=.data$terminalEndOrdinal) %>%
+		dplyr::mutate(
+			currentTraverseOutOrdinal=subjectGr$partnerOrdinal[.data$currentTraverseInOrdinal],
+			distance=subjectGr$insLen[.data$currentTraverseInOrdinal],
+			# TAB is used as a placeholder as it's a disallowed character in VCF and causes a parsing error in BEDPE
+			breakpointsTraversed=paste0(names(subjectGr)[.data$currentTraverseInOrdinal], "	"),
+			traversedDistances=paste0(.data$distance, "	"),
+			traversedBreakpoints=1,
+			minimumTransitiveLength=transitiveGr$minimumTransitiveLength[.data$terminalStartOrdinal],
+			maximumTransitiveLength=transitiveGr$maximumTransitiveLength[.data$terminalStartOrdinal])
+	
+	resultdf = data.frame(
+		terminalStartOrdinal=integer(0),
+		currentTraverseInOrdinal=integer(0),
+		endingTraverseOutOrdinal=integer(0),
+		terminalEndOrdinal=integer(0),
+		currentTraverseOutOrdinal=integer(0),
+		distance=integer(0),
+		ordinalsTraversed=character(0),
+		traversedDistances=character(0),
+		traversedBreakpoints=integer(0))
+	if (nrow(current_traversals > 0)) {
+		traversable_segments = .traversable_segments(subjectGr, max(current_traversals$maximumTransitiveLength))
+		# now we traverse
+		while (nrow(current_traversals) > 0) {
+			# Terminal paths
+			current_traversals = current_traversals %>%
+				dplyr::filter(.data$distance <= .data$maximumTransitiveLength & .data$traversedBreakpoints <= maximumTraversedBreakpoints) %>%
+				dplyr::mutate(is_complete_path=.data$currentTraverseOutOrdinal == .data$endingTraverseOutOrdinal)
+			resultdf = dplyr::bind_rows(
+				resultdf,
+				current_traversals %>% dplyr::filter(
+					.data$is_complete_path &
+					.data$distance >= .data$minimumTransitiveLength &
+					.data$traversedBreakpoints >= minimumTraversedBreakpoints))
+			# traverse
+			current_traversals = current_traversals %>%
+				dplyr::filter(!.data$is_complete_path) %>%
+				dplyr::inner_join(traversable_segments, by=c("currentTraverseInOrdinal"="segmentStartExternalOrdinal")) %>%
+				dplyr::mutate(
+					currentTraverseInOrdinal=.data$segmentEndInternalOrdinal,
+					currentTraverseOutOrdinal=.data$segmentEndExternalOrdinal,
+					distance=.data$distance + .data$segmentLength + .data$segmentEndAdditionalLength,
+					traversedBreakpoints=.data$traversedBreakpoints + 1,
+					traversedDistances=paste0(.data$traversedDistances, .data$distance, "	"),
+					breakpointsTraversed=paste0(.data$breakpointsTraversed, names(subjectGr)[.data$segmentEndInternalOrdinal], "	"))
+			# How can drop columns without a "findTransitiveCalls: no visible binding for global variable 'segmentStartAdditionalLength'" NOTE in R check?
+			#	dplyr::select(
+			#		-segmentStartInternalOrdinal,
+			#		-segmentLength,
+			#		-segmentEndInternalOrdinal,
+			#		-segmentEndExternalOrdinal,
+			#		-segmentStartAdditionalLength,
+			#		-segmentEndAdditionalLength)
+			current_traversals$segmentStartInternalOrdinal = NULL
+			current_traversals$segmentLength = NULL
+			current_traversals$segmentEndInternalOrdinal = NULL
+			current_traversals$segmentEndExternalOrdinal = NULL
+			current_traversals$segmentStartAdditionalLength = NULL
+			current_traversals$segmentEndAdditionalLength = NULL
+		}
+	}
+	# transform results into long form
+	resultdf = resultdf %>%
+		dplyr::mutate(
+			transitive_breakpoint_name=names(transitiveGr)[.data$terminalStartOrdinal],
+			total_distance=as.integer(.data$distance),
+			# trim trailing tab
+			traversed_breakpoint_names=stringr::str_sub(.data$breakpointsTraversed, end=stringr::str_length(.data$breakpointsTraversed) - 1),
+			distance_to_traversed_breakpoint=stringr::str_sub(.data$traversedDistances, end=stringr::str_length(.data$traversedDistances) - 1)) %>%
+		dplyr::select(
+			transitive_breakpoint_name=.data$transitive_breakpoint_name,
+			total_distance=.data$total_distance,
+			traversed_breakpoint_names=.data$traversed_breakpoint_names,
+			distance_to_traversed_breakpoint=.data$distance_to_traversed_breakpoint) %>%
+		as.data.frame() %>%
+		S4Vectors::DataFrame()
+	resultdf$traversed_breakpoint_names = IRanges::CharacterList(stringr::str_split(resultdf$traversed_breakpoint_names, stringr::fixed("	")))
+	resultdf$distance_to_traversed_breakpoint = IRanges::IntegerList(stringr::str_split(resultdf$distance_to_traversed_breakpoint, stringr::fixed("	")))
+	return(resultdf)
+}
+#' @noRd
+.traversable_segments = function(gr, maxgap) {
+	as.data.frame(GenomicRanges::findOverlaps(gr, gr, maxgap=maxgap, ignore.strand=TRUE)) %>%
+		dplyr::filter(
+			(as.logical(strand(gr)[.data$queryHits] == "-") & as.logical(strand(gr)[.data$subjectHits] == "+") & start(gr)[.data$queryHits] <= start(gr)[.data$subjectHits]) |
+				(as.logical(strand(gr)[.data$queryHits] == "+") & as.logical(strand(gr)[.data$subjectHits] == "-") & start(gr)[.data$queryHits] >= start(gr)[.data$subjectHits])) %>%
+		dplyr::select(
+			segmentStartInternalOrdinal=.data$queryHits,
+			segmentEndInternalOrdinal=.data$subjectHits) %>%
+		dplyr::mutate(
+			segmentStartExternalOrdinal=gr$partnerOrdinal[.data$segmentStartInternalOrdinal],
+			segmentEndExternalOrdinal=gr$partnerOrdinal[.data$segmentEndInternalOrdinal],
+			segmentLength=1 + abs(start(gr)[.data$segmentStartInternalOrdinal] - start(gr)[.data$segmentEndInternalOrdinal]),
+			segmentStartAdditionalLength=gr$insLen[.data$segmentStartInternalOrdinal],
+			segmentEndAdditionalLength=gr$insLen[.data$segmentEndInternalOrdinal]) %>%
+		dplyr::select(
+			segmentStartExternalOrdinal=.data$segmentStartExternalOrdinal,
+			segmentStartInternalOrdinal=.data$segmentStartInternalOrdinal,
+			segmentStartAdditionalLength=.data$segmentStartAdditionalLength,
+			segmentLength=.data$segmentLength,
+			segmentEndAdditionalLength=.data$segmentEndAdditionalLength,
+			segmentEndInternalOrdinal=.data$segmentEndInternalOrdinal,
+			segmentEndExternalOrdinal=.data$segmentEndExternalOrdinal)
 }
